<?php
/*
 * User-friendly interface to SIEVE server-side mail filtering.
 * Plugin for Squirrelmail 1.4+
 *
 * Copyright (c) 2002-2003 Alexandros Vellis <avel@users.sourceforge.net>
 *
 * Licensed under the GNU GPL. For full terms see the file COPYING that came
 * with the Squirrelmail distribution.
 *
 */

/**
 * These are just my own wrapper functions around sieve-php.lib.php, with error
 * handling et al.
 */

require_once "sieve-php.lib.php";

/**
 * Upload script
 *
 * @param $newscript str The SIEVE script to be uploaded
 *
 * @return true on success, false upon failure
 */
function avelsieve_upload_script ($newscript) {
	global $sieve;

	/* Uncomment the following for viewing the script that will be
	 * uploaded, for debugging purposes. */
	
	// print "<pre>Sending the script..."; print $newscript; print "</pre>";

	if($sieve->sieve_sendscript("phpscript", stripslashes($newscript))) {
		return true;
		/* print "Your rules have been successfully updated.<br />"; */
	} else {
		print "Unable to load script to server.  See server response below:<br />
		<blockquote><font color=#aa0000>";
		if(isset($sieve->error_raw) && is_array($sieve->error_raw)) {
			foreach($sieve->error_raw as $error_raw)
                        print $error_raw."<br>";
		} else {
                	print $sieve->error_raw."<br>";
		}
		print "</font></blockquote>";
		return false;
	}
}

/**
 * Delete script.
 *
 * Deletes "phpscript" script on SIEVE server
 *
 * @return true on success, false upon failure
 */
function avelsieve_delete_script () {
	global $sieve;

	if($sieve->sieve_deletescript("phpscript")) {
		return true;
	} else {
		print 'Unable to delete script from server. See server response below:<br />
		<blockquote><font color="red">';
		if(is_array($sieve->error_raw)) {
			foreach($sieve->error_raw as $error_raw)
				print $error_raw."<br>";
		} else {
			print $sieve->error_raw."<br>";
		}
	print "</font></blockquote>";
	return false;
	}
}

/**
 * Check if avelsieve capability exists.
 *
 * avelsieve capability is defined as SIEVE capability, NOT'd with
 * $disable_avelsieve_capabilities from configuration file.
 *
 * $disable_avelsieve_capabilities specifies capabilities to disable. If you
 * would like to force avelsieve not to display certain features, even though
 * there _is_ a capability for them by Cyrus/timsieved, you should specify
 * these here. For instance, if you would like to disable the notify extension,
 * even though timsieved advertises it, you should add 'notify' in this array:
 * $force_disable_avelsieve_capability = array("notify");. This will still
 * leave the defined feature on, and if the user can upload her own scripts
 * then she can use that feature; this option just disables the GUI of it.
 *
 * @param $cap capability to check for
 *
 * @return true if capability exists, false if it does not exist
 */
function avelsieve_capability_exists ($cap) {

	global $disable_avelsieve_capabilities, $sieve_capabilities;
	
	if(in_array($cap, $sieve_capabilities)) {
		if(!in_array($cap, $disable_avelsieve_capabilities)) {
			return true;
		}
	}
	return false;
}

?>
